"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const config_1 = require("@cli-engine/config");
const nock = require("nock");
const command_1 = require("../command");
const flags = require("./app");
let mockGitRemotes = jest.fn();
jest.mock('../git', () => ({
    Git: class {
        get remotes() {
            return mockGitRemotes();
        }
    },
}));
let api;
let appfn;
beforeEach(() => {
    mockGitRemotes.mockReturnValue([]);
    api = nock('https://api.heroku.com');
    appfn = jest.fn();
});
afterEach(() => {
    api.done();
});
describe('required', () => {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                appfn(this.flags.app);
            });
        }
    }
    Command.flags = { app: flags.app({ required: true }), remote: flags.remote() };
    test('has an app', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield Command.mock(['--app', 'myapp']);
        expect(appfn).toBeCalledWith('myapp');
    }));
    test('gets app from --remote flag', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        mockGitRemotes.mockReturnValueOnce([
            { name: 'staging', url: 'https://git.heroku.com/myapp-staging.git' },
            { name: 'production', url: 'https://git.heroku.com/myapp-production.git' },
        ]);
        yield Command.mock(['-r', 'staging']);
        expect(appfn).toBeCalledWith('myapp-staging');
    }));
    test('errors if --remote not found', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        expect.assertions(1);
        mockGitRemotes.mockReturnValueOnce([
            { name: 'staging', url: 'https://git.heroku.com/myapp-staging.git' },
            { name: 'production', url: 'https://git.heroku.com/myapp-production.git' },
        ]);
        try {
            yield Command.mock(['-r', 'foo']);
        }
        catch (err) {
            expect(err.message).toEqual('remote foo not found in git remotes');
        }
    }));
    test('errors with no app', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        expect.assertions(1);
        try {
            yield Command.mock();
        }
        catch (err) {
            expect(err.message).toContain('Missing required flag:\n -a, --app');
        }
    }));
    test('errors with 2 git remotes', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        expect.assertions(1);
        mockGitRemotes.mockReturnValueOnce([
            { name: 'staging', url: 'https://git.heroku.com/myapp-staging.git' },
            { name: 'production', url: 'https://git.heroku.com/myapp-production.git' },
        ]);
        try {
            yield Command.mock();
        }
        catch (err) {
            expect(err.message).toContain('Multiple apps in git remotes');
        }
    }));
    test('returns undefined with 2 git remotes when app not required', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        class Command extends command_1.Command {
            run() {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    appfn(this.flags.app);
                });
            }
        }
        Command.flags = { app: flags.app({ required: false }), remote: flags.remote() };
        mockGitRemotes.mockReturnValueOnce([
            { name: 'staging', url: 'https://git.heroku.com/myapp-staging.git' },
            { name: 'production', url: 'https://git.heroku.com/myapp-production.git' },
        ]);
        yield Command.mock();
        expect(appfn).toBeCalledWith(undefined);
    }));
    test('gets app from git config', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        mockGitRemotes.mockReturnValueOnce([{ name: 'heroku', url: 'https://git.heroku.com/myapp.git' }]);
        yield Command.mock();
        expect(appfn).toBeCalledWith('myapp');
    }));
});
describe('optional', () => {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                appfn(this.flags.app);
            });
        }
    }
    Command.flags = { app: flags.app(), remote: flags.remote() };
    test('works when git errors out', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        expect.assertions(1);
        mockGitRemotes.mockImplementationOnce(() => {
            throw new Error('whoa!');
        });
        yield Command.mock();
        expect(appfn).toBeCalledWith(undefined);
    }));
    test('does not error when app is not specified', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield Command.mock();
        expect(appfn).toBeCalledWith(undefined);
    }));
});
describe('completion', () => {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () { });
        }
    }
    Command.flags = { app: flags.app({}) };
    test('cacheDuration defaults to 1 day', () => {
        const completion = Command.flags.app.completion;
        const duration = completion.cacheDuration;
        expect(duration).toEqual(86400);
    });
    test('options returns all the apps', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const completion = Command.flags.app.completion;
        api.get('/apps').reply(200, [{ id: 1, name: 'foo' }, { id: 2, name: 'bar' }]);
        const options = yield completion.options({ config: new config_1.Config() });
        expect(options).toEqual(['bar', 'foo']);
    }));
});
